/*
   Copyright (C) 1988, 1989, 1990 The GAP Development Company

   All Rights Reserved
   
   This source code is considered to be confidential information proprietary
   to The GAP Development Company.  It is prohibited and punishable by law
   to use or duplicate any part of this information without express
   written consent from GAP Development Company.

   This source code is being provided for informational purposes only.


   UTIHIGH.C

   UTI module that obtains the highest message number for an echo mail
   forum and places it into a UTI file.


   To compile : cl /FPa utihigh.c        (will compile and link)

*/


#define LINT_ARGS

#define MULTIUSER
//#define SINGLEUSER

#pragma check_stack(off)
#pragma pack(1)

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <io.h>
#include <share.h>


   int forumnum  = 0;
   long highnum  = 0L;                    // high message # from MESSNO  

   char hismail    [60];
   char dflt_path  [60];                  // path to GAP default dir     


   /**********************************************************************/
   /* A shortened version of GAPBBS.CNF.  All we need is the path to the  /
   /* main directory.                                                     /
   /**********************************************************************/

   struct CONFIG
      {
      char sysname       [16];            // sysop name                   
      char syspas        [13];            // sysop local password         
      int sys_read;                       // level to read private mail   
      int sys_edit;                       // level to edit msg headers    
      int overwrite;                      // level to overwrite uploads   
      int sys_level1;                     // level to view callers log    
      int sys_level2;                     // level to view users          
      int sys_level3;                     // level to view a file         
      int sys_level4;                     // level to maintain user file  
      int sys_level5;                     // level to exit to dos         
      int sys_level5a;                    // level to execute dos funcs   
      int sys_level6;                     // level to view other nodes    
      int sys_level6a;                    // level to log off other node  
      int sys_level7;                     // level to recover messages    

      char answer         [8];            // questionnaire answers file   
      char caller         [7];            // caller file                  
      char doors          [9];            // doors.dat file               
      char forumdat       [9];            // name of forum (.DAT) file    
      char entry_news     [8];            // news file                    
      char exit_news      [8];            // exit news file               
      char pwrd           [9];            // time / security file         
      char fsec           [9];            // files security file          
      char msgs           [9];            // messages file                
      char new_welcome    [8];            // new user welcome file        
      char answer_regis   [8];            // not used                     
      char remote         [9];            // remote.sys file              
      char tcan           [9];            // trashcan file                
      char users          [9];            // users file                   
      char welcome        [8];            // welcome file                 

      char uploads        [26];           // upload directory             
      char gen            [26];           // general directory            
      char maindir        [26];           // main directory               
      }config;


   struct MESSNO
      {  
      long status;                        // multiuser status flags      
      long highmain;                      // highest used message number 
      long highforum [90];                // for main, and each of the   
      };                                  // forums.                     


   static  int pascal open_gapcnf(void );
   static  char *trim(char *str);
   static  int pascal get_high(void );


void main(argc,argv)
int argc;
char *argv[];
{
   int error = 0;

   FILE *fp;

   strcpy(hismail,argv[2]);               // path to HIGHNUM.TXT     
   forumnum = atoi(argv[1]);

   if ( (error = open_gapcnf()) != 0)
      exit (error);

   if ( (error = get_high()) != 0)        // get high message number 
      exit (error);

   if ( (fp = fopen(hismail,"w+t")) == NULL)
      exit (-1);

   fprintf(fp,"%ld\n",highnum);
   
   fclose(fp);

   exit(error);
}

static int pascal open_gapcnf()           // get path to GAP main dir 
{
   int gapfile;

   FILE *fp;


   if ( (fp = fopen("GAPUTI.CNF","rt")) == NULL)
      return (errno);

   fscanf(fp,"%s",dflt_path);             // path to GAP default directory
   fclose(fp);

   strcat(dflt_path,"\\GAPBBS.CNF");

#ifdef MULTIUSER
   gapfile = sopen(dflt_path,O_RDONLY | O_BINARY,SH_DENYNO);
#else
   gapfile = open(dflt_path,O_RDONLY | O_BINARY);
#endif

   if(gapfile == -1)
      return (errno);

   read(gapfile,(char *) &config,sizeof(config));
   close(gapfile);

   trim(config.maindir);

   return (0);
}

static char *trim(str)
char *str;
{
   char *ptr;
   
   ptr = str + strlen(str) -1;
  
   while(ptr >= str && *ptr == ' ')
      *ptr-- = 0;

   return(str);
}

   /****************************************************************/
   /* Open MESSNO.DAT and get the next available message number for /
   /* the echo mail Forum.                                          /
   /****************************************************************/

static int pascal get_high() 
{
   char messpath [60];

   struct MESSNO messno;

   int fp;

   sprintf(messpath,"%s\\MESSNO.DAT",config.maindir);

#ifdef MULTIUSER
   fp = sopen(messpath,O_RDONLY | O_BINARY,SH_DENYNO);
#else
   fp = open(messpath,O_RDONLY | O_BINARY);
#endif

   if (fp == -1)
      return (errno);

   read(fp,(char *) &messno,sizeof(messno));

   highnum = messno.highforum[forumnum-1];

   close(fp);

   return (0);
}
